﻿;-TOP
; Comment     : Include Functions
; Author      : mk-soft
; Version     : 1.01
; Create      : 09.06.2019
; Update      : 

; *****************************************************************************

Macro DebugLine
  Debug #PB_Compiler_Procedure + " / Line " + #PB_Compiler_Line
EndMacro

Macro AddElementText(_List, _Text)
  AddElement(_List)
  _List = _Text
EndMacro

Macro InsertElementText(_List, _Text)
  InsertElement(_List)
  _List = _Text
EndMacro

Macro AddLists(_List_, _Dest_)
  LastElement(_Dest_)
  ForEach _List_
    AddElement(_Dest_)
    _Dest_ = _List_
  Next
EndMacro

; ----

Procedure.s ExtractFunction(String.s)
  
  Structure udtExtractArray
    c.c[0]
  EndStructure
  
  Protected result.s, pos, i, do
  Protected *pString.udtExtractArray
  
  pos = FindString(String, "(")
  If pos
    *pString = @String
    pos - 2
    For i = pos To 0 Step -1
      Select *pString\c[i]
        Case 'A' To 'Z', 'a' To 'z', '0' To '9', '_'
          do = #True
          Continue
        Default
          If do
            i + 1
            pos + 1
            Break
          Else
            pos - 1
          EndIf
      EndSelect
    Next
    i + 1
    pos + 1
    ProcedureReturn Mid(String, i, pos - i)
  Else
    ProcedureReturn ""
  EndIf
EndProcedure

; ----

Procedure SplitParameterList(String.s, List Result.s(), fc=#False)
  Protected *String.character 
  Protected *Start, *End, exit, lock, do, len, temp.s, cnt
  Protected level
  
  ClearList(Result())
  *String = @String
  If *String = 0
    ProcedureReturn 0
  EndIf
  *Start = *String
  *End = *String
  
  Repeat
    If *String\c = 0
      exit = #True
      *End = *String
    Else
      If *String\c = '"'
        If Not lock
          lock = #True
        Else
          lock = #False
        EndIf
      EndIf
      If Not lock
        If *String\c = '('
          If level = 0
            If fc
              AddElement(Result())
              len = (*String - *Start) / SizeOf(character)
              If Len > 0
                temp = Trim(PeekS(*Start, len))
                cnt = CountString(temp, " ")
                If cnt
                  temp = StringField(temp, cnt + 1, " ")
                EndIf
                cnt = CountString(temp, #TAB$)
                If cnt
                  temp = StringField(temp, cnt + 1, #TAB$)
                EndIf
                Result() = temp
              EndIf
            EndIf
            *Start = *String + SizeOf(character)
          EndIf
          level + 1
        ElseIf *String\c = ')'
          level - 1
          If level = 0
            do = #True
            exit = #True
            *End = *String
          EndIf
        ElseIf *String\c = ',' And level = 1
          do = #True
          *End = *String
        EndIf
      EndIf
    EndIf
    If do
      AddElement(Result()) 
      len = (*End - *Start) / SizeOf(character)
      If Len > 0
        Result() = Trim(PeekS(*Start, len))
      EndIf
      *Start = *String + SizeOf(character)
      do = #False
    EndIf
    *String + SizeOf(character)
  Until exit
  
  FirstElement(Result())
  ProcedureReturn ListSize(Result())
  
EndProcedure

; ----

Procedure SplitParameterArray(String.s, Array Result.s(1))
  Protected *String.character 
  Protected *Start, *End, exit, lock, do, len, temp.s, cnt, c1
  Protected level
  
  Dim Result(0)
  
  *String = @String
  If *String = 0
    ProcedureReturn 0
  EndIf
  
  *Start = *String
  *End = *String
  c1 = 0
  
  Repeat
    If *String\c = 0
      exit = #True
      *End = *String
    Else
      If *String\c = '"'
        If Not lock
          lock = #True
        Else
          lock = #False
        EndIf
      EndIf
      If Not lock
        If *String\c = '('
          If level = 0
            If #True
              len = (*String - *Start) / SizeOf(character)
              If Len > 0
                temp = Trim(PeekS(*Start, len))
                cnt = CountString(temp, " ")
                If cnt
                  temp = StringField(temp, cnt + 1, " ")
                EndIf
                cnt = CountString(temp, #TAB$)
                If cnt
                  temp = StringField(temp, cnt + 1, #TAB$)
                EndIf
                Result(c1) = temp
              EndIf
            EndIf
            *Start = *String + SizeOf(character)
          EndIf
          level + 1
        ElseIf *String\c = ')'
          level - 1
          If level = 0
            do = #True
            exit = #True
            *End = *String
          EndIf
        ElseIf *String\c = ',' And level = 1
          do = #True
          *End = *String
        EndIf
      EndIf
    EndIf
    If do
      c1 + 1
      If ArraySize(Result()) < c1
        ReDim Result(c1 + 10)
      EndIf
      len = (*End - *Start) / SizeOf(character)
      If Len > 0
        Result(c1) = Trim(PeekS(*Start, len))
      Else
        Result(c1) = ""
      EndIf
      *Start = *String + SizeOf(character)
      do = #False
    EndIf
    *String + SizeOf(character)
  Until exit
  
  ReDim Result(c1)
  ProcedureReturn c1
  
EndProcedure

; ----

Procedure FindStringLeft(String.s, StringToFind.s, Startposition.l = 0)
  
  Protected len, len2, index
  
  len = Len(String)
  len2 = Len(StringToFind)
  If Startposition = 0
    Startposition = len
  EndIf
  For index = Startposition To 1 Step - 1
    If Mid(String, index, len2) = StringToFind
      ProcedureReturn index
    EndIf
  Next
  ProcedureReturn 0
  
EndProcedure

; ----

Procedure.s ReplaceStringEx(String.s, StringToFind.s, StringToReplace.s)
  
  Protected temp.s, find.s, neu.s, pos, len, ch_left.s, ch_right.s, left_ok, right_ok, lock
  
  temp = UCase(String)
  find = UCase(StringToFind)
  len = Len(find)
  
  neu = String
  pos = 0
  Repeat
    left_ok = #False
    right_ok = #False
    pos + 1
    pos = FindString(temp, find, pos, #PB_String_NoCase)
    If pos
      If pos > 1
        ch_left = Mid(temp, pos - 1, 1)
        If (ch_left < "0" Or ch_left > "9")
          If (ch_left < "A" Or ch_left > "Z")
            If ch_left <> "_"
              left_ok = #True
            EndIf
          EndIf
        EndIf
      Else
        left_ok = #True
      EndIf
      ch_right = Mid(temp, pos + len, 1)
      If ch_right = ""
        right_ok = #True
      Else
        If (ch_right < "0" Or ch_right > "9")
          If (ch_right < "A" Or ch_right > "Z")
            If ch_right <> "_"
              right_ok = #True
            EndIf
          EndIf
        EndIf
      EndIf
      If left_ok And right_ok
        neu = Left(neu, pos - 1 ) + StringToReplace + Mid(neu, pos + len)
        temp = UCase(neu)
        pos + len
      EndIf
    EndIf
  Until pos = 0
  
  ProcedureReturn neu
  
EndProcedure

; ----

Macro FindStringEx(String, StringToFind)
  FindString(String, StringToFind, 1, #PB_String_NoCase)
EndMacro

; ----

Procedure LoadDataSection()
  Protected temp.s
  
  ; Valid Menu Functions
  ClearList(ValidMenus())
  Restore ValidMenus
  Repeat
    Read.s temp.s
    If Bool(temp)
      AddElement(ValidMenus())
      ValidMenus() = LCase(temp)
    Else
      Break
    EndIf
  ForEver
  
  ; Valid Gadget Functions
  ClearList(ValidGadgets())
  Restore ValidGadgets
  Repeat
    Read.s temp.s
    If Bool(temp)
      AddElement(ValidGadgets())
      ValidGadgets() = LCase(temp)
    Else
      Break
    EndIf
  ForEver
  
  ; Valid all Functions
  ClearList(ValidFunctions())
  Restore ValidFunctions
  Repeat
    Read.s temp.s
    If Bool(temp)
      AddElement(ValidFunctions())
      ValidFunctions() = LCase(temp)
    Else
      Break
    EndIf
  ForEver
  
EndProcedure

; ----

Procedure IsValidMenuFunction(Name.s)
  Name = LCase(Name)
  ForEach ValidMenus()
    If ValidMenus() = Name
      ProcedureReturn #True
    EndIf
  Next
  ProcedureReturn #False
EndProcedure

; ----

Procedure IsValidGadgetFunction(Name.s)
  Name = LCase(Name)
  ForEach ValidGadgets()
    If ValidGadgets() = Name
      ProcedureReturn #True
    EndIf
  Next
  ForEach ValidUserGadgets()
    If LCase(ValidUserGadgets()) = Name
      ProcedureReturn #True
    EndIf
  Next
  ProcedureReturn #False
EndProcedure

; ----

Procedure IsValidFunction(Name.s)
  Name = LCase(Name)
  ForEach ValidFunctions()
    If ValidFunctions() = Name
      ProcedureReturn #True
    EndIf
  Next
  ProcedureReturn #False
EndProcedure

; ----

Procedure SearchPrefix(Prefix.s) ; Result Pointer to Form Codes
  Protected *result, temp.s
  temp = LCase(Prefix)
  ForEach Codes()
    ForEach Codes()\ConstWindow()
      If LCase(Codes()\ConstWindow()) = temp
        ProcedureReturn Codes()
      EndIf
    Next
  Next
EndProcedure

; ----

Procedure CheckFormCode(*BaseCodes.udtCodes, List Code.s())
  Protected *Codes.udtCodes
  Protected row.s
  Protected prefix.s, prefix_popup.s, pos, update, temp.s, const.s, const_new.s, const_map.s, proc_name.s
  Protected IsPopupMenu
  
  Dim Params.s(12)
  Dim Params2.s(12)
  
  ForEach Code()
    row = Code()
    update = #False
    
    ;-- Suche Prefix
    If SplitParameterArray(row, Params())
      If Params(0) = "OpenWindow"
        prefix = Params(1)
        If prefix = "#PB_Any"
          WriteLog("! Error: Window Constant '#PB_Any' not supported")
          WriteLog("! Code: " + row)
          ProcedureReturn #False
        EndIf
        If Left(prefix, 1) <> "#"
          WriteLog("! Error: OpenWindow without constants not supported")
          WriteLog("! Code: " + row)
          ProcedureReturn #False
        EndIf
        
      ElseIf Params(0) = "UseGadgetList"
        If SplitParameterArray(Params(1), Params2())
          If Params2(0) <> "WindowID"
            WriteLog("! Error: WindowID without constants not supported")
            WriteLog("! Code: " + row)
            ProcedureReturn #False
          EndIf
          prefix = Params2(1)
          If Left(prefix, 1) <> "#"
            WriteLog("! Error: WindowID without constants not supported")
            WriteLog("! Code: " + row)
            ProcedureReturn #False
          EndIf
          
        EndIf
      EndIf
    EndIf
    
    ;-- Korrigiere Code
    
    ; Menu funktionen
    pos = FindStringEx(row, "CreateMenu(0")
    If pos
      row = ReplaceString(row, "CreateMenu(0", "CreateMenu(#MenuBar", #PB_String_NoCase, pos)
      update = #True
    EndIf
    
    pos = FindStringEx(row, "CreateImageMenu(0")
    If pos
      row = ReplaceString(row, "CreateImageMenu(0", "CreateImageMenu(#MenuBar", #PB_String_NoCase, pos)
      update = #True
    EndIf
    
    ; Statusbar funktionen
    pos = FindStringEx(row, "CreateStatusBar(0")
    If pos
      row = ReplaceString(row, "CreateStatusBar(0", "CreateStatusBar(#StatusBar", #PB_String_NoCase, pos, 1)
      update = #True
    EndIf
    
    pos = FindStringEx(row, "StatusBarText(0")
    If pos
      row = ReplaceString(row, "StatusBarText(0", "StatusBarText(#StatusBar", #PB_String_NoCase, pos)
      update = #True
    EndIf
    
    pos = FindStringEx(row, "StatusBarImage(0")
    If pos
      row = ReplaceString(row, "StatusBarImage(0", "StatusBarImage(#StatusBar", #PB_String_NoCase, pos)
      update = #True
    EndIf
    
    pos = FindStringEx(row, "StatusBarProgress(0")
    If pos
      row = ReplaceString(row, "StatusBarProgress(0", "StatusBarProgress(#StatusBar", #PB_String_NoCase, pos)
      update = #True
    EndIf
    
    pos = FindStringEx(row, "StatusBarHeight(0")
    If pos
      row = ReplaceString(row, "StatusBarHeight(0", "StatusBarHeight(#StatusBar", #PB_String_NoCase, pos)
      update = #True
    EndIf
    
    ; Toolbar funktionen
    pos = FindStringEx(row, "CreateToolBar(0")
    If pos
      row = ReplaceString(row, "CreateToolBar(0", "CreateToolBar(#ToolBar", #PB_String_NoCase, pos, 1)
      update = #True
    EndIf
    
    pos = FindStringEx(row, "ToolBarHeight(0")
    If pos
      row = ReplaceString(row, "ToolBarHeight(0", "ToolBarHeight(#ToolBar", #PB_String_NoCase, pos)
      update = #True
    EndIf
    
    pos = FindStringEx(row, "ToolBarButtonText(0")
    If pos
      row = ReplaceString(row, "ToolBarButtonText(0", "ToolBarButtonText(#ToolBar", #PB_String_NoCase, pos)
      update = #True
    EndIf
    
    pos = FindStringEx(row, "ToolBarToolTip(0")
    If pos
      row = ReplaceString(row, "ToolBarToolTip(0", "ToolBarToolTip(#ToolBar", #PB_String_NoCase, pos)
      update = #True
    EndIf
    
    ; Update LoadImage
    pos = FindStringEx(row, "LoadImage(")
    If pos
      row = Left(row, pos - 1)
      If commonProjectData\Switches\IncludeImages
        row + "CatchImage(" + Params(1) + ", ?Catch_" + Mid(Params(1), 2) + ")"
        update = #True
      ElseIf commonProjectData\Switches\CopyImages
        row + "LoadImage(" + Params(1) + ", "
        row + "__pathResources + " + #DQUOTE$ + GetFilePart(Params(2)) + #DQUOTE$ + ")"
        update = #True
      EndIf
    EndIf
    
    ; Update
    If update
      Code() = row
    EndIf
  Next
  
  With *Codes 
    ;-- Suche Konstanten
    
    *Codes = *BaseCodes
    prefix = *BaseCodes\Prefix
    
    ForEach Code()
      row = Code()
      
      If FindStringEx(row, "Procedure") And Not FindStringEx(row, "EndProcedure")
        proc_name = ExtractFunction(row)
      EndIf
      
      If FindStringEx(row, "Procedure") Or FindStringEx(row, "EndProcedure")
        IsPopupMenu = #False
      EndIf
      
      If SplitParameterArray(row, Params())
        
        If Params(0) = "OpenWindow"
          prefix = Params(1)
          \Prefix = prefix
          const_map = LCase(prefix)
          If Not FindMapElement(ExistsConstant(), const_map)
            AddMapElement(ExistsConstant(), const_map)
            AddElementText(\ConstWindow(), prefix)
            AddElementText(Constants\Windows(), prefix)
            WriteLog("- Add Window Constant " + prefix)
          Else
            Writelog("! - Error: Duplicate Window Constant")
            Writelog("! - Code: " + row)
            ProcedureReturn #False
          EndIf
          
          update = #False
          
          If commonProjectData\Switches\AutoPosition
            Params(2) = "#PB_Ignore"
            Params(3) = "#PB_Ignore"
            update = #True
          EndIf
          
          If codeFlagFirstWindow
            If commonProjectData\Switches\ParendWindow
              If ArraySize(Params()) < 8
                ReDim Params(8)
              EndIf
              If Params(7) = ""
                Params(7) = "0"
              EndIf
              Params(8) = "WindowID(" + codeConstFirstWindow + ")"
              update = #True
            EndIf
          EndIf
          
          If Not codeFlagFirstWindow
            codeFlagFirstWindow = #True
            codeConstFirstWindow = prefix
            codeOpenFirstWindow = proc_name
          EndIf
          
          If update
            pos = FindStringEx(row, "(")
            row = Left(row, pos)
            For pos = 1 To ArraySize(Params())
              row + Params(pos) + ", "
            Next
            row = Left(row, Len(row) - 2) + ")"
            Code() = row
          EndIf
          
        ElseIf Params(0) = "UseGadgetList"
          If SplitParameterArray(Params(1), Params2()) < 1
            WriteLog("! - Error: Count of Parameter")
            Writelog("! - Code: " + row)
            ProcedureReturn #False
          EndIf
          prefix = Params2(1)
          *Codes = SearchPrefix(prefix)
          If Not *Codes
            WriteLog("! - Error: Window Constant not found")
            Writelog("! - Code: " + row)
            ProcedureReturn #False
          EndIf
        
        ElseIf Params(0) = "CreateMenu" Or Params(0) = "CreateImageMenu"
          If Params(1) <> "#PB_Any"
            const = Params(1)
            If SplitParameterArray(Params(2), Params2()) < 1
              WriteLog("! - Error: Count of Parameter")
              Writelog("! - Code: " + row)
              ProcedureReturn #False
            EndIf
            prefix = Params2(1)
            *Codes = SearchPrefix(prefix)
            If Not *Codes
              WriteLog("! - Error: Window Constant not found")
              Writelog("! - Code: " + row)
              ProcedureReturn #False
            Else  
              If Not commonProjectData\Switches\PrefixOff
                const_new = prefix + "_" + Mid(const, 2)
                const_map = LCase(const_new)
                If Not FindMapElement(ExistsConstant(), const_map)
                  AddElement(\ListConst())
                  \ListConst()\Old = const
                  \ListConst()\New = const_new
                  AddMapElement(ExistsConstant(), const_map)
                  AddElementText(\ConstMenus(), const_new)
                  AddElementText(Constants\Menus(), const_new)
                  WriteLog("- Add/Replace MenuBar Constant " + const + " -> " + const_new)
                EndIf
              Else
                const_new = const
                const_map = LCase(const_new)
                If Not FindMapElement(ExistsConstant(), const_map)
                  AddElement(\ListConst())
                  AddMapElement(ExistsConstant(), const_map)
                  AddElementText(\ConstMenus(), const_new)
                  AddElementText(Constants\Menus(), const_new)
                  WriteLog("- Add MenuBar Constant " + const + " -> " + const_new)
                EndIf
              EndIf
            EndIf
          Else
            Writelog("! - Warning: Found #PB_Any")
            Writelog("! - Code: " + row)
          EndIf
        
        ElseIf Params(0) = "CreateStatusBar"
          If Params(1) <> "#PB_Any"
            const = Params(1)
            If SplitParameterArray(Params(2), Params2()) < 1
              WriteLog("! - Error: Count of Parameter")
              Writelog("! - Code: " + row)
              ProcedureReturn #False
            EndIf
            prefix = Params2(1)
            *Codes = SearchPrefix(prefix)
            If Not *Codes
              WriteLog("! - Error: Window Constant not found")
              Writelog("! - Code: " + row)
              ProcedureReturn #False
            Else  
              If Not commonProjectData\Switches\PrefixOff
                const_new = prefix + "_" + Mid(const, 2)
                const_map = LCase(const_new)
                If Not FindMapElement(ExistsConstant(), const_map)
                  AddElement(\ListConst())
                  \ListConst()\Old = const
                  \ListConst()\New = const_new
                  AddMapElement(ExistsConstant(), const_map)
                  AddElementText(\ConstStatusBar(), const_new)
                  AddElementText(Constants\Statusbars(), const_new)
                  WriteLog("- Add/Replace StatusBar Constant " + const + " -> " + const_new)
                EndIf
              Else
                const_new = const
                const_map = LCase(const_new)
                If Not FindMapElement(ExistsConstant(), const_map)
                  AddElement(\ListConst())
                  AddMapElement(ExistsConstant(), const_map)
                  AddElementText(\ConstStatusBar(), const_new)
                  AddElementText(Constants\Statusbars(), const_new)
                  WriteLog("- Add StatusBar Constant " + const + " -> " + const_new)
                EndIf
              EndIf
            EndIf
          Else
            Writelog("! - Warning: Found #PB_Any")
            Writelog("! - Code: " + row)
          EndIf
        
        ElseIf Params(0) = "CreateToolBar"
          If Params(1) <> "#PB_Any"
            const = Params(1)
            If SplitParameterArray(Params(2), Params2()) < 1
              WriteLog("! - Error: Count of Parameter")
              Writelog("! - Code: " + row)
              ProcedureReturn #False
            EndIf
            prefix = Params2(1)
            *Codes = SearchPrefix(prefix)
            If Not *Codes
              WriteLog("! - Error: Window Constant not found")
              Writelog("! - Code: " + row)
              ProcedureReturn #False
            Else
              If Not commonProjectData\Switches\PrefixOff
                const_new = prefix + "_" + Mid(const, 2)
                const_map = LCase(const_new)
                If Not FindMapElement(ExistsConstant(), const_map)
                  AddElement(\ListConst())
                  \ListConst()\Old = const
                  \ListConst()\New = const_new
                  AddMapElement(ExistsConstant(), const_map)
                  AddElementText(\ConstToolbar(), const_new)
                  AddElementText(Constants\Toolbars(), const_new)
                  WriteLog("- Add/Replace ToolBar Constant " + const + " -> " + const_new)
                EndIf
              Else
                const_new = const
                const_map = LCase(const_new)
                If Not FindMapElement(ExistsConstant(), const_map)
                  AddElement(\ListConst())
                  AddMapElement(ExistsConstant(), const_map)
                  AddElementText(\ConstToolbar(), const_new)
                  AddElementText(Constants\Toolbars(), const_new)
                  WriteLog("- Add ToolBar Constant " + const + " -> " + const_new)
                EndIf
              EndIf
            EndIf
          Else
            Writelog("! - Warning: Found #PB_Any")
            Writelog("! - Code: " + row)
          EndIf
        
        ElseIf Params(0) = "CreatePopupMenu" Or Params(0) = "CreatePopupImageMenu"
          ;-- Spezial Popup Menus
          If Params(1) <> "#PB_Any"
            const = Params(1)
            If Left(const, 1) <> "#"
              Writelog("! - Error: Menu without constants not supported")
              Writelog("! - Code: " + row)
              ProcedureReturn #False
            EndIf
            *Codes = *BaseCodes
            IsPopupMenu = #True
            const_new = const
            const_map = LCase(const_new)
            prefix_popup = const
            If Not FindMapElement(ExistsConstant(), const_map)
              AddElement(\ListConst())
              AddMapElement(ExistsConstant(), const_map)
              AddElementText(\ConstMenus(), const_new)
              AddElementText(Constants\Menus(), const_new)
              WriteLog("- Add PopUpMenu Constant " + const_new)
              Code() = ReplaceStringEx(row, const, const_new)
              Continue
            Else
              Writelog("! - Error: Duplicate Menu Constants")
              Writelog("! - Code: " + row)
              ProcedureReturn #False
            EndIf
          Else
            Writelog("! - Warning: Found #PB_Any")
            Writelog("! - Code: " + row)
          EndIf
        EndIf
        
        If *Codes And IsPopupMenu
          ;-- Spezial Popup MenuItems
          If IsValidMenuFunction(Params(0))
            If Params(1) <> "#PB_Any"
              const = Params(1)
              const_new = prefix_popup + "_" + Mid(const, 2)
              const_map = LCase(const_new)
              If Not FindMapElement(ExistsConstant(), const_map)
                AddMapElement(ExistsConstant(), const_map)
                AddElementText(\ConstMenuItems(), const_new)
                AddElementText(Constants\MenuItems(), const_new)
                Code() = ReplaceStringEx(row, const, const_new)
                WriteLog("- Add/Replace Popup MenuItem Constant " + const + " -> " + const_new)
              EndIf
            EndIf
          EndIf
        ElseIf *Codes
          If IsValidMenuFunction(Params(0))
            If Params(1) <> "#PB_Any"
              If Not commonProjectData\Switches\PrefixOff
                const = Params(1)
                const_new = prefix + "_" + Mid(const, 2)
                const_map = LCase(const_new)
                If Not FindMapElement(ExistsConstant(), const_map)
                  AddElement(\ListConst())
                  \ListConst()\Old = const
                  \ListConst()\New = const_new
                  AddMapElement(ExistsConstant(), const_map)
                  AddElementText(\ConstMenuItems(), const_new)
                  AddElementText(Constants\MenuItems(), const_new)
                  WriteLog("- Add/Replace MenuItem Constant " + const + " -> " + const_new)
                EndIf
              Else
                const = Params(1)
                const_new = const
                const_map = LCase(const_new)
                If Not FindMapElement(ExistsConstant(), const_map)
                  AddElement(\ListConst())
                  AddMapElement(ExistsConstant(), const_map)
                  AddElementText(\ConstMenuItems(), const_new)
                  AddElementText(Constants\MenuItems(), const_new)
                  WriteLog("- Add MenuItem Constant " + const + " -> " + const_new)
                EndIf
              EndIf
            Else
              Writelog("! - Warning: Found #PB_Any")
              Writelog("! - Code: " + row)
            EndIf
          EndIf
          
          If IsValidGadgetFunction(Params(0))
            If Params(1) <> "#PB_Any"
              If Not commonProjectData\Switches\PrefixOff
                const = Params(1)
                const_new = prefix + "_" + Mid(const, 2)
                const_map = LCase(const_new)
                If Not FindMapElement(ExistsConstant(), const_map)
                  AddElement(\ListConst())
                  \ListConst()\Old = const
                  \ListConst()\New = const_new
                  AddMapElement(ExistsConstant(), const_map)
                  AddElementText(\ConstGadgets(), const_new)
                  AddElementText(Constants\Gadgets(), const_new)
                  WriteLog("- Add/Replace Gadget Constant " + const + " -> " + const_new)
                EndIf
              Else
                const = Params(1)
                const_new = const
                const_map = LCase(const_new)
                If Not FindMapElement(ExistsConstant(), const_map)
                  AddElement(\ListConst())
                  AddMapElement(ExistsConstant(), const_map)
                  AddElementText(\ConstGadgets(), const_new)
                  AddElementText(Constants\Gadgets(), const_new)
                  WriteLog("- Add/Replace Gadget Constant " + const + " -> " + const_new)
                EndIf
              EndIf
            Else
              Writelog("! - Warning: Found #PB_Any")
              Writelog("! - Code: " + row)
            EndIf
          EndIf
          
          ForEach \ListConst()
            If FindStringEx(row, \ListConst()\Old)
              row = ReplaceStringEx(row, \ListConst()\Old, \ListConst()\New)
              update = #True
            EndIf
          Next
          
          If update
            Code() = row
          EndIf  
          
        EndIf
        
      EndIf
      
    Next
    
  EndWith
  
  ProcedureReturn #True
  
EndProcedure

; ---- 
Procedure LoadForm(Filename.s, List Code.udtCodes())
  Protected result, file, ff, row.s
  Protected isLoadFont, isLoadImage, isProc, isDataSection
  Protected update, temp.s, label.s, pos1, pos2
  
  Dim Params.s(0)
  
  file = ReadFile(#PB_Any, Filename)
  If file
    WriteLog("Load Form: " + filename)
    ;-- Lade Form Datei
    AddElement(Code())
    Code()\Filename = Filename
    
    ff = ReadStringFormat(file)
    While Not Eof(file)
      row = ReadString(file, ff)
      
      If Not isProc
        If FindStringEx(row, "Procedure ") Or FindStringEx(row, "Procedure.")
          AddElement(Code()\Proc())
          isProc = #True
        EndIf
      EndIf
      
      If Not isProc
        If FindStringEx(row, "DataSection")
          isDataSection = #True
        EndIf
      EndIf
      
      If isProc
        AddElement(Code()\Proc()\Rows())
        Code()\Proc()\Rows() = row
        If FindStringEx(row, "EndProcedure")
          isProc = #False
          AddElement(Code()\Proc()\Rows())
          Code()\Proc()\Rows() = ""
        EndIf
      EndIf
      
      If isDataSection
        AddElement(Code()\DataSections())
        Code()\DataSections() = row
        If FindStringEx(row, "EndDataSection")
          isDataSection = #False
          AddElement(Code()\DataSections())
          Code()\DataSections() = ""
        EndIf
      EndIf
      
      pos1 = FindStringEx(row, "LoadFont(")
      If pos1
        temp = Mid(row, pos1)
        AddElement(Code()\LoadFont())
        Code()\LoadFont()\Function = Trim(temp)
        If SplitParameterArray(temp, Params()) < 2
          WriteLog("! - Error: Count of Parameter")
          Writelog("! - Code: " + row)
          ProcedureReturn #False
        EndIf
        Code()\LoadFont()\Const = Params(1)
        Code()\LoadFont()\FontName = Params(2)
        Code()\LoadFont()\IsInside = isProc
      EndIf
      
      pos1 = FindStringEx(row, "LoadImage(")
      If pos1
        temp = Mid(row, pos1)
        AddElement(Code()\LoadImage())
        Code()\LoadImage()\Function = Trim(temp)
        If SplitParameterArray(temp, Params()) < 2
          WriteLog("! - Error: Count of Parameter")
          Writelog("! - Code: " + row)
          ProcedureReturn #False
        EndIf
        Code()\LoadImage()\Const = Params(1)
        Code()\LoadImage()\FileName = Params(2)
        Code()\LoadImage()\IsInside = isProc
      EndIf
      
      pos1 = FindStringEx(row, "CatchImage(")
      If pos1
        temp = Mid(row, pos1)
        AddElement(Code()\CatchImage())
        Code()\CatchImage()\Function = Trim(temp)
        If SplitParameterArray(temp, Params()) < 2
          WriteLog("! - Error: Count of Parameter")
          Writelog("! - Code: " + row)
          ProcedureReturn #False
        EndIf
        Code()\CatchImage()\Const = Params(1)
        Code()\CatchImage()\Label = Mid(Params(2), 2) + ":"
        Code()\CatchImage()\IsInside = isProc
      EndIf
      
      If Not isProc
        
        If FindString(row, "Decoder()")
          temp = ExtractFunction(row)
          UseImageDecoder(temp) = temp + "()"
        EndIf
        
        If FindStringEx(row, "Global ")
          AddElement(Code()\Globals())
          Code()\Globals() = row
        EndIf
        
      EndIf
      
    Wend
    CloseFile(file)
    
    ;-- Check Procedure Code
    ForEach Code()\Proc()
      If Not CheckFormCode(Code(), Code()\Proc()\Rows())
        ProcedureReturn #False
      EndIf
    Next
    
    ;-- Suche CatchImage Filenames
    ForEach Code()\CatchImage()
      label.s = Code()\CatchImage()\Label
      ForEach Code()\DataSections()
        If FindStringEx(Code()\DataSections(), label)
          Repeat
            pos1 = FindStringEx(Code()\DataSections(), "IncludeBinary")
            If pos1
              pos1 = FindString(Code()\DataSections(), #DQUOTE$, pos1)
              pos2 = FindString(Code()\DataSections(), #DQUOTE$, pos1 + 1)
              Code()\CatchImage()\FileName = Mid(Code()\DataSections(), pos1, pos2 - pos1 + 1)
              Break 2
            EndIf
          Until NextElement(Code()\DataSections()) = 0
        EndIf
      Next
    Next
    
    ;-- Fonts eintragen
    ForEach Code()\LoadFont()
      If Code()\LoadFont()\Const <> "#PB_Any"
        AddElement(Constants\LoadFonts())
        Constants\LoadFonts() = Code()\LoadFont()
      EndIf
    Next
    
    ;-- Load Images eintragen
    ForEach Code()\LoadImage()
      If Code()\LoadImage()\Const <> "#PB_Any"
        AddElement(Constants\LoadImages())
        Constants\LoadImages() = Code()\LoadImage()
      EndIf
    Next
    
    ;-- Catch Images eintragen
    ForEach Code()\CatchImage()
      If Code()\CatchImage()\Const <> "#PB_Any"
        AddElement(Constants\CatchImages())
        Constants\CatchImages() = Code()\CatchImage()
      EndIf
    Next
    
    ;-- Ende Form Datei
    ProcedureReturn #True
    
  EndIf
  
EndProcedure

; ----

Procedure LoadProjectFiles()
  Protected file, filename.s, ff, temp.s
  
  ;-- Lade Main File
  ClearList(Files\Main())
  filename = commonProgramData\ProjectPath + "GuiMainFile.pb"
  file = ReadFile(#PB_Any, filename)
  If file
    Writelog("- Load Main File " + filename)
    ff = ReadStringFormat(file)
    If ff = 0
      ff = #PB_Ascii
    EndIf
    While Not Eof(file)
      temp = ReadString(file, ff)
      AddElementText(Files\Main(), temp)
    Wend
    CloseFile(file)
  EndIf
  
  ;-- Lade Event Files
  ClearList(Files\EventFiles())
  ForEach commonProjectData\Files()
    filename = commonProgramData\ProjectPath + "GuiEvent" + GetFilePart(commonProjectData\Files(), #PB_FileSystem_NoExtension) + ".pb"
    file = ReadFile(#PB_Any, filename)
    If file
      AddElement(Files\EventFiles())
      Files\EventFiles()\FileName = filename
      Files\EventFiles()\Update = #False
      Writelog("- Load Event File " + filename)
      ff = ReadStringFormat(file)
      If ff = 0
        ff = #PB_Ascii
      EndIf
      While Not Eof(file)
        temp = ReadString(file, ff)
        AddElementText(Files\EventFiles()\Code(), temp)
      Wend
      CloseFile(file)
    EndIf
  Next
  
  ProcedureReturn #True
  
EndProcedure

; ----

Procedure LoadIncludeFiles(List Code.s(), List Result.udtCode())
  
  Protected IFile.s, Line.s, Path.s, Pos, PosB, PosE, sTemp.s, found
  Protected file, filename.s, ff, filetemp.s
  
  path = commonProgramData\ProjectPath
  
  ForEach Code()
    IFile = ""
    
    Line = Code()
    
    Pos = FindString(Line, "IncludePath", 1)
    If FindStringLeft(Line, ";", Pos) Or FindStringLeft(Line, #DQUOTE$, Pos)
      Pos = 0
    EndIf
    If Pos
      PosB = FindString(Line, #DQUOTE$, Pos + 12) + 1
      PosE = FindString(Line, #DQUOTE$, PosB)
      If PosB And PosE
        Path = Mid(Line, PosB, PosE - PosB)
        If path = ""
          path = commonProgramData\ProjectPath
        Else
          If Right(path, 1) <> #PS$
            path + #PS$
          EndIf
        EndIf
      EndIf
    EndIf
    
    Pos = FindStringEx(Line, "IncludeFile")
    If FindStringLeft(Line, ";", Pos) Or FindStringLeft(Line, #DQUOTE$, Pos)
      Pos = 0
    EndIf
    If Pos
      PosB = FindString(Line, #DQUOTE$, Pos + 12) + 1
      PosE = FindString(Line, #DQUOTE$, PosB)
      If PosB And PosE
        IFile = Mid(Line, PosB, PosE - PosB)
      EndIf
    EndIf
    
    sTemp = LCase(IFile)
    If sTemp = "guicommonfile.pb"
      Continue
    EndIf
    If sTemp = "guieventmainfile.pb"
      Continue
    EndIf
    
    If IFile <> ""
      If GetPathPart(IFile) = ""
        sTemp = path + IFile
      Else
        sTemp = IFile
      EndIf
      found = #False
      ForEach Result()
        If LCase(Result()\FileName) = LCase(stemp)
          found = #True
          Break
        EndIf
      Next
      If Not found
        ; Add Include File
        AddElement(Result())
        Result()\FileName = sTemp
        ; Load Include File
        filename = Result()\FileName
        file = ReadFile(#PB_Any, filename)
        If file
          Writelog("- Load Include File " + filename)
          ff = ReadStringFormat(file)
          If ff = 0
            ff = #PB_Ascii
          EndIf
          While Not Eof(file)
            filetemp = ReadString(file, ff)
            AddElementText(Result()\Code(), filetemp)
          Wend
          CloseFile(file)
        Else
          Writelog("! - Error: Load Include File")
          Writelog("! - File: " + filename)
          ProcedureReturn #False
        EndIf
        ; Recursive
        PushListPosition(Result())
        If Not LoadIncludeFiles(Result()\Code(), Files\IncludeFiles())
          ProcedureReturn #False
        EndIf
        PopListPosition(Result())
      EndIf
      
    EndIf
  Next
  
  ProcedureReturn #True
  
EndProcedure

; ----

Procedure CompareWindow(Title.s, List Old.s(), List New.s(), List Update.udtOldNew())
  Protected const.s, found
  
  CompareConstants\Cancel = #False
  CompareConstants\Title = Title
  ClearList(CompareConstants\Old())
  ClearList(CompareConstants\New())
  ClearList(CompareConstants\Update())
  
  ; Suche alte Konstanten
  ForEach Old()
    const = LCase(Old())
    found = #False
    ForEach New()
      If LCase(New()) = const
        found = #True
        Break
      EndIf
    Next
    If Not found
      AddElementText(CompareConstants\Old(), Old())
    EndIf
  Next
  ; Suche neu Konstanten
  ForEach New()
    const = LCase(New())
    found = #False
    ForEach Old()
      If LCase(Old()) = const
        found = #True
        Break
      EndIf
    Next
    If Not found
      AddElementText(CompareConstants\New(), New())
    EndIf
  Next
  
  If ListSize(CompareConstants\Old())
    PostEvent(#MyEventInitCompareWindow)
    WaitSemaphore(SignalCompareWindow)
    If CompareConstants\Cancel
      ProcedureReturn #False
    EndIf
    If ListSize(CompareConstants\Update())
      MergeLists(CompareConstants\Update(), Update())
    EndIf
  EndIf
  
  ProcedureReturn #True
  
EndProcedure

; ----

Procedure InitCompareWindow()
  OpenWinCheck()
  If IsWindow(#WinCheck)
    SetWindowTitle(#WinCheck, CompareConstants\Title)
    ForEach CompareConstants\Old()
      AddGadgetItem(#WinCheck_Listview_Old, -1, CompareConstants\Old())
    Next
    ForEach CompareConstants\New()
      AddGadgetItem(#WinCheck_Listview_New, -1, CompareConstants\New())
    Next
  Else
    CompareConstants\Cancel = #True
    SignalSemaphore(SignalCompareWindow)
  EndIf
EndProcedure

BindEvent(#MyEventInitCompareWindow, @InitCompareWindow())

; ----

Procedure CompareConstants()
  Protected const_old.s, const_new.s
  
  CopyList(commonProjectData\Constants\Windows(), OldConstants\Windows())
  CopyList(commonProjectData\Constants\Menus(), OldConstants\Menus())
  CopyList(commonProjectData\Constants\MenuItems(), OldConstants\MenuItems())
  CopyList(commonProjectData\Constants\Gadgets(), OldConstants\Gadgets())
  CopyList(commonProjectData\Constants\Toolbars(), OldConstants\Toolbars())
  CopyList(commonProjectData\Constants\Statusbars(), OldConstants\Statusbars())
  
  ClearList(UpdateConstants\Windows())
  ClearList(UpdateConstants\Menus())
  ClearList(UpdateConstants\MenuItems())
  ClearList(UpdateConstants\Gadgets())
  ClearList(UpdateConstants\Toolbars())
  ClearList(UpdateConstants\Statusbars())
  
  ; Vergleiche Window Konstanten
  If Not CompareWindow("Window Constants", OldConstants\Windows(), Constants\Windows(), UpdateConstants\Windows())
    ProcedureReturn #False
  EndIf
  
  ; Aktuallisiere alte Window Konstanten vor Vergleich
  ForEach UpdateConstants\Windows()
    const_old = UpdateConstants\Windows()\Old + "_"
    const_new = UpdateConstants\Windows()\New + "_"
    ForEach OldConstants\Menus()
      If FindStringEx(OldConstants\Menus(), const_old)
        AddElement(UpdateConstants\Menus())
        UpdateConstants\Menus()\Old = OldConstants\Menus()
        OldConstants\Menus() = ReplaceString(OldConstants\Menus(), const_old, const_new)
        UpdateConstants\Menus()\New = OldConstants\Menus()
      EndIf
    Next
    ForEach OldConstants\MenuItems()
      If FindStringEx(OldConstants\MenuItems(), const_old)
        AddElement(UpdateConstants\MenuItems())
        UpdateConstants\MenuItems()\Old = OldConstants\MenuItems()
        OldConstants\MenuItems() = ReplaceString(OldConstants\MenuItems(), const_old, const_new)
        UpdateConstants\MenuItems()\New = OldConstants\MenuItems()
      EndIf
    Next
    ForEach OldConstants\Gadgets()
      If FindStringEx(OldConstants\Gadgets(), const_old)
        AddElement(UpdateConstants\Gadgets())
        UpdateConstants\Gadgets()\Old = OldConstants\Gadgets()
        OldConstants\Gadgets() = ReplaceString(OldConstants\Gadgets(), const_old, const_new)
        UpdateConstants\Gadgets()\New = OldConstants\Gadgets()
      EndIf
    Next
    ForEach OldConstants\Toolbars()
      If FindStringEx(OldConstants\Toolbars(), const_old)
        AddElement(UpdateConstants\Toolbars())
        UpdateConstants\Toolbars()\Old = OldConstants\Toolbars()
        OldConstants\Toolbars() = ReplaceString(OldConstants\Toolbars(), const_old, const_new)
        UpdateConstants\Toolbars()\New = OldConstants\Toolbars()
      EndIf
    Next
    ForEach OldConstants\Statusbars()
      If FindStringEx(OldConstants\Statusbars(), const_old)
        AddElement(UpdateConstants\Statusbars())
        UpdateConstants\Statusbars()\Old = OldConstants\Statusbars()
        OldConstants\Statusbars() = ReplaceString(OldConstants\Statusbars(), const_old, const_new)
        UpdateConstants\Statusbars()\New = OldConstants\Statusbars()
      EndIf
    Next
  Next
  
  ; Vergleiche MenuBar Konstanten
  If Not CompareWindow("MenuBar Constants", OldConstants\Menus(), Constants\Menus(), UpdateConstants\Menus())
    ProcedureReturn #False
  EndIf
  
  ; Aktuallisiere alte Menu Konstanten vor Vergleich
  ForEach UpdateConstants\Menus()
    const_old = UpdateConstants\Menus()\Old + "_"
    const_new = UpdateConstants\Menus()\New + "_"
    ForEach OldConstants\MenuItems()
      If FindStringEx(OldConstants\MenuItems(), const_old)
        AddElement(UpdateConstants\MenuItems())
        UpdateConstants\MenuItems()\Old = OldConstants\MenuItems()
        OldConstants\MenuItems() = ReplaceString(OldConstants\MenuItems(), const_old, const_new)
        UpdateConstants\MenuItems()\New = OldConstants\MenuItems()
      EndIf
    Next
  Next
  
  ; Vergleiche MenuItem Konstanten
  If Not CompareWindow("MenuItem Constants", OldConstants\MenuItems(), Constants\MenuItems(), UpdateConstants\MenuItems())
    ProcedureReturn #False
  EndIf
  
  ; Vergleiche Gadget Konstanten
  If Not CompareWindow("Gadget Constants", OldConstants\Gadgets(), Constants\Gadgets(), UpdateConstants\Gadgets())
    ProcedureReturn #False
  EndIf
  
  ; Vergleiche ToolBar Konstanten
  If Not CompareWindow("ToolBar Constants", OldConstants\Toolbars(), Constants\Toolbars(), UpdateConstants\Toolbars())
    ProcedureReturn #False
  EndIf
  
  ; Vergleiche Menubar Konstanten
  If Not CompareWindow("StatusBar Constants", OldConstants\Statusbars(), Constants\Statusbars(), UpdateConstants\Statusbars())
    ProcedureReturn #False
  EndIf
  
  ProcedureReturn #True
  
EndProcedure

; ----

Procedure UpdateWindowConstants()
  Protected i
  
  Dim Update.udtOldNew(4)
  
  ForEach UpdateConstants\Windows()
    
    Update(0)\Old = UpdateConstants\Windows()\Old
    Update(0)\New = UpdateConstants\Windows()\New
    Update(1)\Old = "Open" + Mid(UpdateConstants\Windows()\Old, 2) + "()"
    Update(1)\New = "Open" + Mid(UpdateConstants\Windows()\New, 2) + "()"
    Update(2)\Old = "ResizeGadgets" + Mid(UpdateConstants\Windows()\Old, 2)
    Update(2)\New = "ResizeGadgets" + Mid(UpdateConstants\Windows()\New, 2)
    Update(3)\Old = "MyEventSizeWindow_" + Mid(UpdateConstants\Windows()\Old, 2) + "()"
    Update(3)\New = "MyEventSizeWindow_" + Mid(UpdateConstants\Windows()\New, 2) + "()"
    Update(4)\Old = "MyEventCloseWindow_" + Mid(UpdateConstants\Windows()\Old, 2) + "()"
    Update(4)\New = "MyEventCloseWindow_" + Mid(UpdateConstants\Windows()\New, 2) + "()"
    
    ForEach Files\Main()
      For i = 0 To 1
        If FindStringEx(Files\Main(), Update(i)\Old)
          Files\UpdateMain = #True
          Files\Main() = ReplaceStringEx(Files\Main(), Update(i)\Old, Update(i)\New)
        EndIf
      Next
    Next
    
    ForEach Files\EventFiles()
      ForEach Files\EventFiles()\Code()
        For i = 0 To 4
          If FindStringEx(Files\EventFiles()\Code(), Update(i)\Old)
            Files\EventFiles()\Update = #True
            Files\EventFiles()\code() = ReplaceStringEx(Files\EventFiles()\Code(), Update(i)\Old, Update(i)\New)
          EndIf
        Next
      Next
    Next
    
    ForEach Files\IncludeFiles()
      ForEach Files\IncludeFiles()\Code()
        For i = 0 To 1
          If FindStringEx(Files\IncludeFiles()\Code(), Update(i)\Old)
            Files\IncludeFiles()\Update = #True
            Files\IncludeFiles()\Code() = ReplaceStringEx(Files\IncludeFiles()\Code(), Update(i)\Old, Update(i)\New)
          EndIf
        Next
      Next
    Next
    
  Next
  
EndProcedure

; ----

Procedure UpdateMenuBarConstants()
  Protected const_old.s, const_new.s
  
  ForEach UpdateConstants\Menus()
    const_old = UpdateConstants\Menus()\Old
    const_new = UpdateConstants\Menus()\New
    
    ForEach Files\Main()
      If FindStringEx(Files\Main(), const_old)
        Files\UpdateMain = #True
        Files\Main() = ReplaceStringEx(Files\Main(), const_old, const_new)
      EndIf
    Next
    
    ForEach Files\EventFiles()
      ForEach Files\EventFiles()\Code()
        If FindStringEx(Files\EventFiles()\Code(), const_old)
          Files\EventFiles()\Update = #True
          Files\EventFiles()\Code() = ReplaceStringEx(Files\EventFiles()\Code(), const_old, const_new)
        EndIf
      Next
    Next
    
    ForEach Files\IncludeFiles()
      ForEach Files\IncludeFiles()\Code()
        If FindStringEx(Files\IncludeFiles()\Code(), const_old)
          Files\IncludeFiles()\Update = #True
          Files\IncludeFiles()\Code() = ReplaceStringEx(Files\IncludeFiles()\Code(), const_old, const_new)
        EndIf
      Next
    Next
    
  Next
  
EndProcedure

; ----

Procedure UpdateMenuItemConstants()
  Protected i
  
  Dim Update.udtOldNew(1)
  
  ForEach UpdateConstants\MenuItems()
    Update(0)\Old = UpdateConstants\MenuItems()\Old
    Update(0)\New = UpdateConstants\MenuItems()\New
    Update(1)\Old = "MyEventMenu_" + Mid(UpdateConstants\MenuItems()\Old, 2) + "()"
    Update(1)\New = "MyEventMenu_" + Mid(UpdateConstants\MenuItems()\New, 2) + "()"
    
    ForEach Files\Main()
      For i = 0 To 0
        If FindStringEx(Files\Main(), Update(i)\Old)
          Files\UpdateMain = #True
          Files\Main() = ReplaceStringEx(Files\Main(), Update(i)\Old, Update(i)\New)
        EndIf
      Next
    Next
    
    ForEach Files\EventFiles()
      ForEach Files\EventFiles()\Code()
        For i = 0 To 1
          If FindStringEx(Files\EventFiles()\Code(), Update(i)\Old)
            Files\EventFiles()\Update = #True
            Files\EventFiles()\code() = ReplaceStringEx(Files\EventFiles()\Code(), Update(i)\Old, Update(i)\New)
          EndIf
        Next
      Next
    Next
    
    ForEach Files\IncludeFiles()
      ForEach Files\IncludeFiles()\Code()
        For i = 0 To 0
          If FindStringEx(Files\IncludeFiles()\Code(), Update(i)\Old)
            Files\IncludeFiles()\Update = #True
            Files\IncludeFiles()\Code() = ReplaceStringEx(Files\IncludeFiles()\Code(), Update(i)\Old, Update(i)\New)
          EndIf
        Next
      Next
    Next
    
  Next
  
EndProcedure

; ----

Procedure UpdateGadgetConstants()
  Protected i
  
  Dim Update.udtOldNew(1)
  
  ForEach UpdateConstants\Gadgets()
    Update(0)\Old = UpdateConstants\Gadgets()\Old
    Update(0)\New = UpdateConstants\Gadgets()\New
    Update(1)\Old = "MyEventGadget_" + Mid(UpdateConstants\Gadgets()\Old, 2) + "()"
    Update(1)\New = "MyEventGadget_" + Mid(UpdateConstants\Gadgets()\New, 2) + "()"
    
    ForEach Files\Main()
      For i = 0 To 0
        If FindStringEx(Files\Main(), Update(i)\Old)
          Files\UpdateMain = #True
          Files\Main() = ReplaceStringEx(Files\Main(), Update(i)\Old, Update(i)\New)
        EndIf
      Next
    Next
    
    ForEach Files\EventFiles()
      ForEach Files\EventFiles()\Code()
        For i = 0 To 1
          If FindStringEx(Files\EventFiles()\Code(), Update(i)\Old)
            Files\EventFiles()\Update = #True
            Files\EventFiles()\code() = ReplaceStringEx(Files\EventFiles()\Code(), Update(i)\Old, Update(i)\New)
          EndIf
        Next
      Next
    Next
    
    ForEach Files\IncludeFiles()
      ForEach Files\IncludeFiles()\Code()
        For i = 0 To 0
          If FindStringEx(Files\IncludeFiles()\Code(), Update(i)\Old)
            Files\IncludeFiles()\Update = #True
            Files\IncludeFiles()\Code() = ReplaceStringEx(Files\IncludeFiles()\Code(), Update(i)\Old, Update(i)\New)
          EndIf
        Next
      Next
    Next
    
  Next
  
EndProcedure

; ----

Procedure UpdateToolBarConstants()
  Protected const_old.s, const_new.s
  
  ForEach UpdateConstants\Toolbars()
    const_old = UpdateConstants\Toolbars()\Old
    const_new = UpdateConstants\Toolbars()\New
    
    ForEach Files\Main()
      If FindStringEx(Files\Main(), const_old)
        Files\UpdateMain = #True
        Files\Main() = ReplaceStringEx(Files\Main(), const_old, const_new)
      EndIf
    Next
    
    ForEach Files\EventFiles()
      ForEach Files\EventFiles()\Code()
        If FindStringEx(Files\EventFiles()\Code(), const_old)
          Files\EventFiles()\Update = #True
          Files\EventFiles()\Code() = ReplaceStringEx(Files\EventFiles()\Code(), const_old, const_new)
        EndIf
      Next
    Next
    
    ForEach Files\IncludeFiles()
      ForEach Files\IncludeFiles()\Code()
        If FindStringEx(Files\IncludeFiles()\Code(), const_old)
          Files\IncludeFiles()\Update = #True
          Files\IncludeFiles()\Code() = ReplaceStringEx(Files\IncludeFiles()\Code(), const_old, const_new)
        EndIf
      Next
    Next
    
  Next
  
EndProcedure

; ----

Procedure UpdateStatusBarConstants()
  Protected const_old.s, const_new.s
  
  ForEach UpdateConstants\Statusbars()
    const_old = UpdateConstants\Statusbars()\Old
    const_new = UpdateConstants\Statusbars()\New
    
    ForEach Files\Main()
      If FindStringEx(Files\Main(), const_old)
        Files\UpdateMain = #True
        Files\Main() = ReplaceStringEx(Files\Main(), const_old, const_new)
      EndIf
    Next
    
    ForEach Files\EventFiles()
      ForEach Files\EventFiles()\Code()
        If FindStringEx(Files\EventFiles()\Code(), const_old)
          Files\EventFiles()\Update = #True
          Files\EventFiles()\Code() = ReplaceStringEx(Files\EventFiles()\Code(), const_old, const_new)
        EndIf
      Next
    Next
    
    ForEach Files\IncludeFiles()
      ForEach Files\IncludeFiles()\Code()
        If FindStringEx(Files\IncludeFiles()\Code(), const_old)
          Files\IncludeFiles()\Update = #True
          Files\IncludeFiles()\Code() = ReplaceStringEx(Files\IncludeFiles()\Code(), const_old, const_new)
        EndIf
      Next
    Next
    
  Next
  
EndProcedure

; ----

Procedure CreateCommonFile()
  Protected temp.s, filename.s, doLine
  
  WriteLog("- Create Common File")
  
  With Files
    AddElementText(\Common(), ";-TOP")
    AddElementText(\Common(), "")
    AddElementText(\Common(), "; ***************************************************************************")
    AddElementText(\Common(), "; ")
    AddElementText(\Common(), "; Comment : EventDesigner - Common Form File")
    AddElementText(\Common(), "; Create  : " + FormatDate("%YYYY.%MM.%DD %HH.%II.%SS", Date()))
    AddElementText(\Common(), "; ")
    AddElementText(\Common(), "; Info    : Do not edit this file. This file always overwrite.")
    AddElementText(\Common(), "; ")
    AddElementText(\Common(), "; ***************************************************************************")
    AddElementText(\Common(), "")
    
    AddElementText(\Common(), ";- Bugfix ResizeGadet with BindEvent and CreateMenu")
    AddElementText(\Common(), "")
    AddElementText(\Common(), "Macro _PB_(Function)")
    AddElementText(\Common(), "  Function")
    AddElementText(\Common(), "EndMacro")
    AddElementText(\Common(), "")
    
    AddElementText(\Common(), "Macro ResizeGadget(Gadget, x, y, Width, Height)")
    AddElementText(\Common(), "  If IsGadget(Gadget) : _PB_(ResizeGadget)(Gadget, x, y, Width, Height) : EndIf")
    AddElementText(\Common(), "EndMacro")
    AddElementText(\Common(), "")
    AddElementText(\Common(), "CompilerIf #PB_Compiler_OS = #PB_OS_Linux")
    AddElementText(\Common(), "  Macro ToolBarHeight(ToolBar)")
    AddElementText(\Common(), "    0")
    AddElementText(\Common(), "  EndMacro")
    AddElementText(\Common(), "CompilerEndIf")
    AddElementText(\Common(), "")
    
    AddElementText(\Common(), ";- Enumerations")
    AddElementText(\Common(), "")
    
    AddElementText(\Common(), "Enumeration FormWindow")
    ForEach Constants\Windows()
      AddElementText(\Common(), "  " + Constants\Windows())
    Next
    AddElementText(\Common(), "EndEnumeration")
    AddElementText(\Common(), "")
    
    If ListSize(Constants\Menus())
      AddElementText(\Common(), "Enumeration FormMenuBar")
      ForEach Constants\Menus()
        AddElementText(\Common(), "  " + Constants\Menus())
      Next
      AddElementText(\Common(), "EndEnumeration")
      AddElementText(\Common(), "")
    EndIf
    
    If ListSize(Constants\Toolbars())
      AddElementText(\Common(), "Enumeration FormToolBar")
      ForEach Constants\Toolbars()
        AddElementText(\Common(), "  " + Constants\Toolbars())
      Next
      AddElementText(\Common(), "EndEnumeration")
      AddElementText(\Common(), "")
    EndIf
    
    If ListSize(Constants\Statusbars())
      AddElementText(\Common(), "Enumeration FormStatusBar")
      ForEach Constants\Statusbars()
        AddElementText(\Common(), "  " + Constants\Statusbars())
      Next
      AddElementText(\Common(), "EndEnumeration")
      AddElementText(\Common(), "")
    EndIf
    
    If ListSize(Constants\MenuItems())
      AddElementText(\Common(), "Enumeration FormMenu")
      ForEach Constants\MenuItems()
        AddElementText(\Common(), "  " + Constants\MenuItems())
      Next
      AddElementText(\Common(), "EndEnumeration")
      AddElementText(\Common(), "")
    EndIf
    
    If ListSize(Constants\Gadgets())
      AddElementText(\Common(), "Enumeration FormGadget")
      ForEach Constants\Gadgets()
        AddElementText(\Common(), "  " + Constants\Gadgets())
      Next
      AddElementText(\Common(), "EndEnumeration")
      AddElementText(\Common(), "")
    EndIf
    
    If ListSize(Constants\LoadFonts())
      AddElementText(\Common(), "Enumeration FormFont")
      ForEach Constants\LoadFonts()
        AddElementText(\Common(), "  " + Constants\LoadFonts()\Const)
      Next
      AddElementText(\Common(), "EndEnumeration")
      AddElementText(\Common(), "")
    EndIf
    
    If ListSize(Constants\LoadImages()) Or ListSize(Constants\CatchImages())
      AddElementText(\Common(), "Enumeration FormImage")
      ForEach Constants\LoadImages()
        AddElementText(\Common(), "  " + Constants\LoadImages()\Const)
      Next
      ForEach Constants\CatchImages()
        AddElementText(\Common(), "  " + Constants\CatchImages()\Const)
      Next
      AddElementText(\Common(), "EndEnumeration")
      AddElementText(\Common(), "")
    EndIf
    
    If ListSize(Constants\LoadFonts())
      AddElementText(\Common(), ";- Load Fonts")
      AddElementText(\Common(), "")
      
      ForEach Constants\LoadFonts()
        If Not Constants\LoadFonts()\IsInside
          AddElementText(\Common(), Constants\LoadFonts()\Function)
        EndIf
      Next
      AddElementText(\Common(), "")
    EndIf
    
    If MapSize(UseImageDecoder())
      AddElementText(\Common(), ";- Use Image Decoder")
      AddElementText(\Common(), "")
      ForEach UseImageDecoder()
        AddElementText(\Common(), UseImageDecoder())
      Next
      AddElementText(\Common(), "")
    EndIf
    
    If ListSize(Constants\LoadImages()) Or ListSize(Constants\CatchImages())
      AddElementText(\Common(), ";- Load Images")
      AddElementText(\Common(), "")
      
      If commonProjectData\Switches\IncludeImages
        ForEach Constants\LoadImages()
          temp = "CatchImage(" + Constants\LoadImages()\Const + ", ?Catch_" + Mid(Constants\LoadImages()\Const, 2) + ")"
          AddElementText(\Common(), temp)
        Next
      Else
        If commonProjectData\Switches\CopyImages
          CompilerSelect #PB_Compiler_OS
            CompilerCase #PB_OS_MacOS
              temp = commonProgramData\ProjectPath + "MyAppData/"
              If FileSize(temp) <> -2
                CreateDirectory(temp)
              EndIf
              temp = commonProgramData\ProjectPath + "MyAppData/Resources/"
              If FileSize(temp) <> -2
                CreateDirectory(temp)
              EndIf
            CompilerCase #PB_OS_Windows
              temp = commonProgramData\ProjectPath + "Resources\"
              If FileSize(temp) <> -2
                CreateDirectory(temp)
              EndIf
            CompilerCase #PB_OS_Linux
              temp = commonProgramData\ProjectPath + "Resources/"
              If FileSize(temp) <> -2
                CreateDirectory(temp)
              EndIf
          CompilerEndSelect
          WriteLog("-- Created Image Directory " + temp)
          ForEach Constants\LoadImages()
            filename = RemoveString(Constants\LoadImages()\FileName, #DQUOTE$)
            If CopyFile(filename , temp + GetFilePart(filename))
              WriteLog("--- Copy Image " + filename)
            Else
              WriteLog("!-- Error Copy Image " + filename)
            EndIf
          Next
          
          AddElementText(\Common(), "CompilerIf #PB_Compiler_OS = #PB_OS_MacOS")
          AddElementText(\Common(), "  Global __pathProgram.s = GetPathPart(ProgramFilename())")
          AddElementText(\Common(), "  Global __pathResources.s = Left(__pathProgram, Len(__pathProgram) - 6) + " + #DQUOTE$ + "Resources/" + #DQUOTE$)
          AddElementText(\Common(), "CompilerElse")
          AddElementText(\Common(), "  Global __pathProgram.s = GetPathPart(ProgramFilename())")
          AddElementText(\Common(), "  Global __pathResources.s = __pathProgram + " + #DQUOTE$ + "Resources" + #DQUOTE$ + " + #PS$")
          AddElementText(\Common(), "CompilerEndIf")
          AddElementText(\Common(), "")
          
          ForEach Constants\LoadImages()
            If Not Constants\LoadImages()\IsInside
              AddElementText(\Common(), "LoadImage(" + Constants\LoadImages()\Const + ", __pathResources + " + #DQUOTE$ + GetFilePart(Constants\LoadImages()\FileName) + #DQUOTE$ + ")")
            EndIf
          Next
        Else
          ForEach Constants\LoadImages()
            If Not Constants\LoadImages()\IsInside
              AddElementText(\Common(), Constants\LoadImages()\Function)
            EndIf
          Next
        EndIf
      EndIf
      
      ForEach Constants\CatchImages()
        If Not Constants\CatchImages()\IsInside
          AddElementText(\Common(), Constants\CatchImages()\Function)
        EndIf
      Next
        
      AddElementText(\Common(), "")
    EndIf
    
    AddElementText(\Common(), "; ----")
    AddElementText(\Common(), "")
    
    ForEach Codes()
      AddElementText(\Common(), ";- Form: " + Codes()\Filename)
      AddElementText(\Common(), "")
      
      If ListSize(Codes()\Globals())
        ForEach Codes()\Globals()
          AddElementText(\Common(), Codes()\Globals())
        Next
        AddElementText(\Common(), "")
      EndIf
      
      doLine = #False
      
      If ListSize(Codes()\LoadFont())
        ForEach Codes()\LoadFont()
          If Codes()\LoadFont()\Const = "#PB_Any"
            AddElementText(\Common(), Codes()\LoadFont()\Function)
            doLine = #True
          EndIf
        Next
      EndIf
      
      If doLine
        AddElementText(\Common(), "")
      EndIf
      
      doLine = #False
      
      If ListSize(Codes()\LoadImage())
        ForEach Codes()\LoadImage()
          If Codes()\LoadImage()\Const = "#PB_Any"
            AddElementText(\Common(), Codes()\LoadImage()\Function)
            doLine = #True
          EndIf
        Next
      EndIf
      
      If ListSize(Codes()\CatchImage())
        ForEach Codes()\CatchImage()
          If Codes()\CatchImage()\Const = "#PB_Any"
            AddElementText(\Common(), Codes()\CatchImage()\Function)
            doLine = #True
          EndIf
        Next
      EndIf
      
      If doLine
        AddElementText(\Common(), "")
      EndIf
      
      ForEach Codes()\Proc()
        ForEach Codes()\Proc()\Rows()
          AddElementText(\Common(), Codes()\Proc()\Rows())
        Next
      Next
      
      ForEach Codes()\DataSections()
        AddElementText(\Common(), Codes()\DataSections())
      Next
      
      AddElementText(\Common(), "; ----")
      AddElementText(\Common(), "")
      
    Next
    
    AddElementText(\Common(), "UndefineMacro ResizeGadget")
    AddElementText(\Common(), "UndefineMacro ToolBarHeight")
    AddElementText(\Common(), "")
    AddElementText(\Common(), "; ----")
    AddElementText(\Common(), "")
    
    If ListSize(Constants\LoadImages())
      If commonProjectData\Switches\IncludeImages
        AddElementText(\Common(), ";- DataSection Images ")
        AddElementText(\Common(), "")
        AddElementText(\Common(), "DataSection")
        ForEach Constants\LoadImages()
          temp = "  Catch_" + Mid(Constants\LoadImages()\Const, 2) + ":"
          AddElementText(\Common(), temp)
          temp = "  IncludeBinary " + Constants\LoadImages()\FileName
          AddElementText(\Common(), temp)
        Next
        AddElementText(\Common(), "EndDataSection")
        AddElementText(\Common(), "")
      EndIf
    EndIf
    
    ProcedureReturn #True
    
  EndWith
EndProcedure

; ----

Procedure CreateMainFile()
  Protected temp.s, start, ende, len
  
  If ListSize(Files\Main()) = 0
    Writelog("- Create Main File")
    temp = PeekS(?CodeMainFile, -1, #PB_UTF8)
    temp = RemoveString(temp, #CR$)
    start = 1
    While start
      ende = FindString(temp, #LF$, start)
      len = ende - start
      If len >= 0
        AddElementText(Files\Main(), Mid(temp, start, len))
        start = ende + 1
      Else
        Break
      EndIf
    Wend
    
    ForEach Files\Main()
      Files\Main() = ReplaceString(Files\Main(), "OpenWindowXY", codeOpenFirstWindow, #PB_String_NoCase)
    Next
    
    ForEach Files\Main()
      Files\Main() = ReplaceStringEx(Files\Main(), "#WindowXY", codeConstFirstWindow)
    Next
    
    Files\UpdateMain = #True
    
  EndIf
  
EndProcedure

; ----

Procedure AddEventMenuMacOS(List Code.s())
  
  AddElementText(Code(), "Procedure MyEventMenu_MacAbout()")
  AddElementText(Code(), "  ")
  AddElementText(Code(), "EndProcedure")
  AddElementText(Code(), "")
  AddElementText(Code(), "; ----")
  AddElementText(Code(), "")
  AddElementText(Code(), "Procedure MyEventMenu_MacPreferences()")
  AddElementText(Code(), "  ")
  AddElementText(Code(), "EndProcedure")
  AddElementText(Code(), "")
  AddElementText(Code(), "; ----")
  AddElementText(Code(), "")
  AddElementText(Code(), "Procedure MyEventMenu_MacQuit()")
  AddElementText(Code(), "  ExitApplication = 1")
  AddElementText(Code(), "EndProcedure")
  AddElementText(Code(), "")
  AddElementText(Code(), "; ----")
  AddElementText(Code(), "")
  
EndProcedure

; ----

Procedure CreateEventFiles()
  Protected file, filename.s, found 
  Protected temp.s, first, first_menu, start, ende, update
  Protected const.s, func.s
  
  first = #False
  first_menu = #False
  
  Writelog("- Create Event Files")
  
  ForEach Codes()
    ;-- Event Datei suchen
    filename = commonProgramData\ProjectPath + "GuiEvent" + GetFilePart(Codes()\Filename, #PB_FileSystem_NoExtension) + ".pb"
    found = #False
    ForEach Files\EventFiles()
      If Files\EventFiles()\FileName = filename
        found = #True
        Break
      EndIf
    Next
    
    If Not found
      AddElement(Files\EventFiles())
      Files\EventFiles()\FileName = filename
    EndIf
    
    update = #False
    
    With Files\EventFiles()
      
      If ListSize(\Code()) = 0
        
        ;-- Event Datei neu erstellen
        
        Writelog("  - New Event File " + Files\EventFiles()\FileName)
        
        ;--- New Header
        AddElementText(\Code(), ";-TOP")
        AddElementText(\Code(), "")
        AddElementText(\Code(), "; ***************************************************************************")
        AddElementText(\Code(), ";")
        AddElementText(\Code(), "; Comment : EventDesigner - Event Form File")
        AddElementText(\Code(), "; Create  : " + FormatDate("%YYYY.%MM.%DD %HH:%II:%SS", Date()))
        AddElementText(\Code(), "; Update  : ")
        AddElementText(\Code(), ";")
        AddElementText(\Code(), "; ***************************************************************************")
        AddElementText(\Code(), "")
        
        ;--- New ResizeWindow
        ForEach Codes()\ConstWindow()
          const = Codes()\ConstWindow()
          func = Mid(Codes()\ConstWindow(), 2)
          AddElementText(\Code(), "Procedure MyEventSizeWindow_" + func + "()")
          AddElementText(\Code(), "  CompilerIf Defined(ResizeGadgets" + func + ", #PB_Procedure)")
          AddElementText(\Code(), "    ResizeGadgets" + func + "()")
          AddElementText(\Code(), "  CompilerEndIf")
          AddElementText(\Code(), "EndProcedure")
          AddElementText(\Code(), "")
          AddElementText(\Code(), "; ----")
          AddElementText(\Code(), "")
          update = #True
        Next
        
        ;--- New CloseWindow
        ForEach Codes()\ConstWindow()
          const = Codes()\ConstWindow()
          func = Mid(Codes()\ConstWindow(), 2) + "()"
          AddElementText(\Code(), "Procedure MyEventCloseWindow_" + func)
          If Not first
            first = #True
            AddElementText(\Code(), "  ExitApplication = #True")
          Else
            AddElementText(\Code(), "  CloseWindow(" + const + ")")
          EndIf
          AddElementText(\Code(), "EndProcedure")
          AddElementText(\Code(), "")
          AddElementText(\Code(), "; ----")
          AddElementText(\Code(), "")
          update = #True
        Next
        
        ;--- New MacOS Menu Events
        If Not first_menu
          first_menu = #True
          If commonProjectData\Switches\MacMenuEvents
            AddEventMenuMacOS(\Code())
            update = #True
          EndIf
        EndIf
        
        ;--- New Menu Events
        ForEach Codes()\ConstMenuItems()
          const = Codes()\ConstMenuItems()
          func = Mid(Codes()\ConstMenuItems(), 2) + "()"
          AddElementText(\Code(), "Procedure MyEventMenu_" + func)
          AddElementText(\Code(), "  ")
          AddElementText(\Code(), "EndProcedure")
          AddElementText(\Code(), "")
          AddElementText(\Code(), "; ----")
          AddElementText(\Code(), "")
          update = #True
        Next
        
        ;--- New Gadget Events
        ForEach Codes()\ConstGadgets()
          const = Codes()\ConstGadgets()
          func = Mid(Codes()\ConstGadgets(), 2) + "()"
          AddElementText(\Code(), "Procedure MyEventGadget_" + func)
          AddElementText(\Code(), "  ")            
          AddElementText(\Code(), "  Select EventType()")
          AddElementText(\Code(), "    Case #PB_EventType_LeftClick")
          AddElementText(\Code(), "      ")
          AddElementText(\Code(), "  EndSelect")
          AddElementText(\Code(), "  ")            
          AddElementText(\Code(), "EndProcedure")
          AddElementText(\Code(), "")
          AddElementText(\Code(), "; ----")
          AddElementText(\Code(), "")
          update = #True
        Next
        
        ;--- Update Datei
        Files\EventFiles()\Update = #True
        first_menu = #True
        
      Else
        
        ;-- Event Datei aktuallisieren
        
        #PBEndCode = "; IDE Options = PureBasic"
        
        ;--- Header aktuallisieren
        
        temp = "; Update  : " + FormatDate("%YYYY.%MM.%DD %HH:%II:%SS", Date())
        ForEach \Code()
          If FindStringEx(\Code(), "; Update")
            \Code() = temp
            Break
          EndIf
          If FindStringEx(\Code(), #PBEndCode)
            Break
          EndIf
        Next
        
        ;--- Add SizeWindow
        
        ForEach Codes()\ConstWindow()
          const = Codes()\ConstWindow()
          func = "MyEventSizeWindow_" + Mid(Codes()\ConstWindow(), 2) + "()"
          found = #False
          start = 0
          ForEach \Code()
            If Not start And FindStringEx(\Code(), "Procedure MyEventCloseWindow")
              start = ListIndex(\Code())
            EndIf
            
            If Not start And FindStringEx(\Code(), "Procedure MyEventMenu")
              start = ListIndex(\Code())
            EndIf
            
            If Not start And FindStringEx(\Code(), "Procedure MyEventGadget")
              start = ListIndex(\Code())
            EndIf
            
            If Not start And FindStringEx(\Code(), #PBEndCode)
              Start = ListIndex(\Code())
            EndIf
            
            If FindStringEx(\Code(), func)
              found = #True
              Break
            EndIf
          Next ; Code Zeile
          
          If Not found
            If start > 1
              SelectElement(\Code(), start)
            Else
              LastElement(\Code())
              AddElementText(\Code(), "")
            EndIf
            
            func = Mid(Codes()\ConstWindow(), 2)
            InsertElementText(\Code(), "Procedure MyEventSizeWindow_" + func + "()")
            AddElementText(\Code(), "  CompilerIf Defined(ResizeGadgets" + func + ", #PB_Procedure)")
            AddElementText(\Code(), "    ResizeGadgets" + func + "()")
            AddElementText(\Code(), "  CompilerEndIf")
            AddElementText(\Code(), "EndProcedure")
            AddElementText(\Code(), "")
            AddElementText(\Code(), "; ----")
            AddElementText(\Code(), "")
            update = #True
          EndIf  
          
        Next ; Konstante
        
        ;--- Add CloseWindow
        
        ForEach Codes()\ConstWindow()
          const = Codes()\ConstWindow()
          func = "MyEventCloseWindow_" + Mid(Codes()\ConstWindow(), 2) + "()"
          found = #False
          start = 0
          ForEach \Code()
            If Not start And FindStringEx(\Code(), "Procedure MyEventMenu")
              start = ListIndex(\Code())
            EndIf
            
            If Not start And FindStringEx(\Code(), "Procedure MyEventGadget")
              start = ListIndex(\Code())
            EndIf
            
            If Not start And FindStringEx(\Code(), #PBEndCode)
              Start = ListIndex(\Code())
            EndIf
            
            If FindStringEx(\Code(), func)
              found = #True
              Break
            EndIf
          Next ; Code Zeile
          
          If Not found
            If start > 1
              SelectElement(\Code(), start)
            Else
              LastElement(\Code())
              AddElementText(\Code(), "")
            EndIf
            
            const = Codes()\ConstWindow()
            func = Mid(Codes()\ConstWindow(), 2) + "()"
            InsertElementText(\Code(), "Procedure MyEventCloseWindow_" + func)
            If const = codeConstFirstWindow
              AddElementText(\Code(), "  ExitApplication = #True")
            Else
              AddElementText(\Code(), "  CloseWindow(" + const + ")")
            EndIf
            AddElementText(\Code(), "EndProcedure")
            AddElementText(\Code(), "")
            AddElementText(\Code(), "; ----")
            AddElementText(\Code(), "")
            update = #True
          EndIf 
          
        Next ; Konstante
        
        ;--- Add Event Menu MacOS
        
        If Not first_menu
          first_menu = #True
          If commonProjectData\Switches\MacMenuEvents
            func = "MyEventMenu_MacQuit()"
            found = #False
            start = 0
            ForEach \Code()
              If Not Start And FindStringEx(\Code(), "Procedure MyEventMenu")
                start = ListIndex(\Code())
              EndIf
              
              If Not Start And FindStringEx(\Code(), "Procedure MyEventGadget")
                start = ListIndex(\Code())
              EndIf
              
              If Not start And FindStringEx(\Code(), #PBEndCode)
                start = ListIndex(\Code())
              EndIf
              
              If FindStringEx(\Code(), func)
                found = #True
                Break
              EndIf
            Next ; Code Zeile
            
            If Not found
              If start > 1
                SelectElement(\Code(), start)
              Else
                LastElement(\Code())
                AddElementText(\Code(), "")
              EndIf
              If start > 0
                PreviousElement(\Code())
              EndIf
              AddEventMenuMacOS(\Code())
              update = #True
            EndIf
            
          EndIf
        EndIf
        
        ;--- Add EventMenu
        
        ForEach Codes()\ConstMenuItems()
          const = Codes()\ConstMenuItems()
          func = "MyEventMenu_" + Mid(Codes()\ConstMenuItems(), 2) + "()"
          found = #False
          start = 0
          ForEach \Code()
            If Not start And FindStringEx(\Code(), "Procedure MyEventGadget")
              start = ListIndex(\Code())
            EndIf
            
            If Not start And FindStringEx(\Code(), #PBEndCode)
              Start = ListIndex(\Code())
            EndIf
            
            If FindStringEx(\Code(), func)
              found = #True
              Break
            EndIf
          Next ; Code Zeile
          
          If Not found
            If start > 1
              SelectElement(\Code(), start)
            Else
              LastElement(\Code())
              AddElementText(\Code(), "")
            EndIf
            
            InsertElementText(\Code(), "Procedure " + func)
            AddElementText(\Code(), "  ")
            AddElementText(\Code(), "EndProcedure")
            AddElementText(\Code(), "")
            AddElementText(\Code(), "; ----")
            AddElementText(\Code(), "")
            update = #True
          EndIf
          
        Next ; Konstante
        
        ;--- Add EventGadget
        
        ForEach Codes()\ConstGadgets()
          const = Codes()\ConstGadgets()
          func = "MyEventGadget_" + Mid(Codes()\ConstGadgets(), 2) + "()"
          found = #False
          start = 0
          ForEach \Code()
            
            If Not start And FindStringEx(\Code(), #PBEndCode)
              Start = ListIndex(\Code())
            EndIf
            
            If FindStringEx(\Code(), func)
              found = #True
              Break
            EndIf
          Next ; Code Zeile
          
          If Not found
            If start > 1
              SelectElement(\Code(), start)
            Else
              LastElement(\Code())
              AddElementText(\Code(), "")
            EndIf
            
            InsertElementText(\Code(), "Procedure " + func)
            AddElementText(\Code(), "  ")            
            AddElementText(\Code(), "  Select EventType()")
            AddElementText(\Code(), "    Case #PB_EventType_LeftClick")
            AddElementText(\Code(), "      ")
            AddElementText(\Code(), "  EndSelect")
            AddElementText(\Code(), "  ")            
            AddElementText(\Code(), "EndProcedure")
            AddElementText(\Code(), "")
            AddElementText(\Code(), "; ----")
            AddElementText(\Code(), "")
            update = #True
          EndIf
          
        Next ; Konstante
        
      EndIf
      
      ;--- Update Datei
      If update
        Writelog("  - Update Event File " + Files\EventFiles()\FileName)
        Files\EventFiles()\Update = #True
      EndIf
      first = #True
      first_menu = #True
      
    EndWith
    
  Next ; Datei
  
  ProcedureReturn #True
  
EndProcedure

; ----

Procedure CreateEventMainFile()
  Protected temp.s, start, ende, len
  
  ClearList(Files\EventMain())
  
  If ListSize(Files\EventMain()) = 0
    Writelog("- Create Event Main File")
    If commonProjectData\Switches\MacMenuEvents
      temp = PeekS(?CodeMainEventFileMacOS, -1, #PB_UTF8)
    Else
      temp = PeekS(?CodeMainEventFile, -1, #PB_UTF8)
    EndIf  
    temp = RemoveString(temp, #CR$)
    start = 1
    While start
      ende = FindString(temp, #LF$, start)
      len = ende - start
      If len >= 0
        AddElementText(Files\EventMain(), Mid(temp, start, len))
        start = ende + 1
      Else
        Break
      EndIf
    Wend
    
    temp = "; Create  : " + FormatDate("%YYYY.%MM.%DD %HH.%II.%SS", Date())
    ForEach Files\EventMain()
      If FindStringEx(Files\EventMain(), "; Create")
        Files\EventMain() =  temp
        Break
      EndIf
    Next
    
    ForEach Files\EventMain()
      If FindStringEx(Files\EventMain(), "; - Include Event Files")
        ForEach Files\EventFiles()
          AddElement(Files\EventMain())
          ;Files\EventMain() = "IncludeFile " + #DQUOTE$ + Files\EventFiles()\FileName + #DQUOTE$
          Files\EventMain() = "IncludeFile " + #DQUOTE$ + GetFilePart(Files\EventFiles()\FileName) + #DQUOTE$
        Next
      EndIf
    Next
    
    temp = "#_CountEventWindow = " + ListSize(Constants\Windows())
    ForEach Files\EventMain()
      If FindStringEx(Files\EventMain(), "#_CountEventWindow")
        Files\EventMain() =  temp
        Break
      EndIf
    Next
    
    temp = "#_CountEventMenu = " + ListSize(Constants\MenuItems())
    ForEach Files\EventMain()
      If FindStringEx(Files\EventMain(), "#_CountEventMenu")
        Files\EventMain() =  temp
        Break
      EndIf
    Next
    
    temp = "#_CountEventGadget = " + ListSize(Constants\Gadgets())
    ForEach Files\EventMain()
      If FindStringEx(Files\EventMain(), "#_CountEventGadget")
        Files\EventMain() =  temp
        Break
      EndIf
    Next
    
    ForEach Files\EventMain()
      If FindString(Files\EventMain(), "DataSection")
        Break
      EndIf
    Next
    
    AddElementText(Files\EventMain(), "  vTableEventSizeWindow:")
    ForEach Constants\Windows()
      AddElementText(Files\EventMain(), "  Data.i @MyEventSizeWindow_" + Mid(Constants\Windows(), 2) + "()")
    Next
    AddElementText(Files\EventMain(), "  ")
    
    AddElementText(Files\EventMain(), "  vTableEventCloseWindow:")
    ForEach Constants\Windows()
      AddElementText(Files\EventMain(), "  Data.i @MyEventCloseWindow_" + Mid(Constants\Windows(), 2) + "()")
    Next
    AddElementText(Files\EventMain(), "  ")
    
    AddElementText(Files\EventMain(), "  vTableEventMenu:")
    ForEach Constants\MenuItems()
      AddElementText(Files\EventMain(), "  Data.i @MyEventMenu_" + Mid(Constants\MenuItems(), 2) + "()")
    Next
    AddElementText(Files\EventMain(), "  ")
    
    AddElementText(Files\EventMain(), "  vTableEventGadget:")
    ForEach Constants\Gadgets()
      AddElementText(Files\EventMain(), "  Data.i @MyEventGadget_" + Mid(Constants\Gadgets(), 2) + "()")
    Next
    AddElementText(Files\EventMain(), "  ")
    
    ProcedureReturn #True
  EndIf
  
EndProcedure

; ----
Procedure MakeBackup()
  
  Protected path.s, path_copy.s
  
  path = commonProgramData\ProjectPath
  path_copy = path + "Backup"
  CreateDirectory(path_copy)
  If CopyDirectory(path, path_copy, "*.pb*")
    CopyDirectory(path, path_copy, "*.xml")
  Else
    Writelog("! - Error: CopyDirectory")
    Writelog("! - Source: " + path)
    Writelog("! - Dest: " + path_copy)
    ProcedureReturn #False
  EndIf
  ProcedureReturn #True
  
EndProcedure

; ----

Procedure WriteProjectFiles()
  Protected file, filename.s
  
  filename = commonProgramData\ProjectPath + "GuiCommonFile.pb"
  file = CreateFile(#PB_Any, filename)
  If file
    WriteLog("- Write File: " + filename)
    WriteStringFormat(file, #PB_UTF8)
    ForEach Files\Common()
      WriteStringN(file, Files\Common(), #PB_UTF8)
    Next
    CloseFile(file)
  Else
    WriteLog("! - Error: Write File: " + filename)
    ProcedureReturn #False
  EndIf
  
  If Files\UpdateMain
    filename = commonProgramData\ProjectPath + "GuiMainFile.pb"
    file = CreateFile(#PB_Any, filename)
    If file
      WriteLog("- Write File: " + filename)
      WriteStringFormat(file, #PB_UTF8)
      ForEach Files\Main()
        WriteStringN(file, Files\Main(), #PB_UTF8)
      Next
      CloseFile(file)
    Else
      WriteLog("! - Error: Write File: " + filename)
      ProcedureReturn #False
    EndIf
  EndIf
  
  filename = commonProgramData\ProjectPath + "GuiEventMainFile.pb"
  file = CreateFile(#PB_Any, filename)
  If file
    WriteLog("- Write File: " + filename)
    WriteStringFormat(file, #PB_UTF8)
    ForEach Files\EventMain()
      WriteStringN(file, Files\EventMain(), #PB_UTF8)
    Next
    CloseFile(file)
  Else
    WriteLog("! - Error: Write File: " + filename)
    ProcedureReturn #False
  EndIf
  
  ForEach Files\EventFiles()
    If Files\EventFiles()\Update
      filename = Files\EventFiles()\FileName
      file = CreateFile(#PB_Any, filename)
      If file
        WriteLog("- Write File: " + filename)
        WriteStringFormat(file, #PB_UTF8)
        ForEach Files\EventFiles()\Code()
          WriteStringN(file, Files\EventFiles()\Code(), #PB_UTF8)
        Next
        CloseFile(file)
      Else
        WriteLog("! - Error: Write File: " + filename)
        ProcedureReturn #False
      EndIf
    EndIf
  Next
  
  ForEach Files\IncludeFiles()
    If Files\IncludeFiles()\Update
      filename = Files\IncludeFiles()\FileName
      file = CreateFile(#PB_Any, filename)
      If file
        WriteLog("- Write File: " + filename)
        WriteStringFormat(file, #PB_UTF8)
        ForEach Files\IncludeFiles()\Code()
          WriteStringN(file, Files\IncludeFiles()\Code(), #PB_UTF8)
        Next
        CloseFile(file)
      Else
        WriteLog("! - Error: Write File: " + filename)
        ProcedureReturn #False
      EndIf
    EndIf
  Next
  
  ProcedureReturn #True
  
EndProcedure

; ----

Procedure thWork(*Data.udtThreadControl)
  
  With *Data
    ;-- Init
    ThreadToGUI::DoClearGadgetItems(#Main_Listview_Status)
    
    If commonProjectData\Switches\CloseIDE
      
      WriteLog("Wait: Close Purebasic IDE ...")
      
      If CloseNamedWindow("PureBasic", #True)
        Delay(200)
        While FindNamedWindow("PureBasic", #True)
          If \Exit
            WriteLog("! - Stop: Cancel by User")
            ProcedureReturn 0
          EndIf
          Delay(1000)
        Wend
      EndIf
    EndIf
    
    ThreadToGUI::DoDisableWindow(#Main, #True)
    
    WriteLog("Start ...")
    
    ;-- Listen löschen
    ClearList(Codes())
    
    ClearMap(ExistsConstant())
    
    ClearList(Constants\Windows())
    ClearList(Constants\Menus())
    ClearList(Constants\MenuItems())
    ClearList(Constants\Toolbars())
    ClearList(Constants\Statusbars())
    ClearList(Constants\Gadgets())
    ClearList(Constants\LoadFonts())
    ClearList(Constants\LoadImages())
    ClearList(Constants\CatchImages())
    
    ClearList(Files\Common())
    
    codeFlagFirstWindow = #False
    codeConstFirstWindow = ""
    codeOpenFirstWindow = ""
    codeResizeFirstWindow = ""
    
    Files\UpdateMain = #False
    Files\UpdateEventMain = #False
    Files\UpdateCommon = #False
    
    Repeat ; Until #True
      
      ;-- Forms laden
      
      WriteLog("Load Forms ...")
      ForEach commonProjectData\Files()
        ; 1. Query on thread pause
        If \Pause
          WaitSemaphore(\Signal)
        EndIf
        ; 2. Query on thread cancel
        If \Exit
          Break
        EndIf
        If Not LoadForm(commonProjectData\Files(), Codes())
          Break 2
        EndIf
      Next
      
      ;-- Lade Projekt Dateien
      WriteLog("Load Project Files ...")
      If Not LoadProjectFiles()
        Break
      EndIf
      
      ;-- Lade Include Files
      WriteLog("Load Include Files ...")
      ClearList(Files\IncludeFiles())
      If Not LoadIncludeFiles(Files\Main(), Files\IncludeFiles())
        ProcedureReturn #False
      EndIf
      
      ;-- Überprüfe Konstanten
      WriteLog("Check Constants ...")
      If Not CompareConstants()
        WriteLog("! - Stop: Cancel by User")
        Break
      EndIf
      
      If ListSize(UpdateConstants\Windows())
        Writelog("- Update Window Constants")
        UpdateWindowConstants()
      EndIf
      
      If ListSize(UpdateConstants\Menus())
        Writelog("- Update MenuBar Constants")
        UpdateMenuBarConstants()
      EndIf
      
      If ListSize(UpdateConstants\MenuItems())
        Writelog("- Update MenuItem Constants")
        UpdateMenuItemConstants()
      EndIf
      
      If ListSize(UpdateConstants\Gadgets())
        Writelog("- Update Gadget Constants")
        UpdateGadgetConstants()
      EndIf
      
      If ListSize(UpdateConstants\Toolbars())
        Writelog("- Update ToolBar Constants")
        UpdateToolBarConstants()
      EndIf
      
      If ListSize(UpdateConstants\Statusbars())
        Writelog("- Update StatusBar Constants")
        UpdateStatusBarConstants()
      EndIf
      
      ;-- Erstelle Code
      WriteLog("Create Codes ...")
      
      CreateCommonFile()
      CreateMainFile()
      CreateEventFiles()
      CreateEventMainFile()
      
      ;-- Sichere Projekt Dateien
      WriteLog("Copy old Files ...")
      If Not MakeBackup()
        Break
      EndIf
      
      ;-- Schreibe Projekt Dateien
      WriteLog("Write Files ...")
      If Not WriteProjectFiles()
        Break
      EndIf
      
      ;-- Projekt Daten
      WriteLog("Result ...")
      WriteLog("- Count Windows = " + ListSize(Constants\Windows()))
      WriteLog("- Count Menubars = " + ListSize(Constants\Menus()))
      WriteLog("- Count MenuItems = " + ListSize(Constants\MenuItems()))
      WriteLog("- Count Gadgets = " + ListSize(Constants\Gadgets()))
      WriteLog("- Count Toolbars = " + ListSize(Constants\Toolbars()))
      WriteLog("- Count Statusbars = " + ListSize(Constants\Statusbars()))
      WriteLog("- Count Fonts = " + ListSize(Constants\LoadFonts()))
      WriteLog("- Count Images = " + Str(ListSize(Constants\LoadImages()) + ListSize(Constants\CatchImages())))
      
      ;-- Alle Konstanten übernehmen
      commonProjectData\Constants = Constants
      flagProjectDataSave = #False
      
      Writelog("Ready.")
      
      ; Purebasic IDE neu starten
      If commonProjectData\Switches\CloseIDE
        CompilerSelect #PB_Compiler_OS
          CompilerCase #PB_OS_MacOS
            RunProgram("Open", commonProgramData\ProjectPath + "GuiMainFile.pb", "")
          CompilerCase #PB_OS_Windows
            ;RunProgram("Open", commonProgramData\ProjectPath + "GuiMainFile.pb", "")
            RunProgram(commonProgramData\ProjectPath + "GuiMainFile.pb")
          CompilerCase #PB_OS_Linux
            RunProgram("Open", commonProgramData\ProjectPath + "GuiMainFile.pb", "")
            
        CompilerEndSelect
      Else
        Writelog("Warning! Check the Purebasic-IDE have reload all project files!")
      EndIf
      
      ThreadToGUI::DoDisableWindow(#Main, #False)
      \ThreadID = 0
      
      ProcedureReturn 1
      
    Until #True
    
    Writelog("Cancel.")
    
    ThreadToGUI::DoDisableWindow(#Main, #False)
    \ThreadID = 0
    
  EndWith
  
EndProcedure

; ----

Procedure StartProcess()
  ClearGadgetItems(#Main_Listview_Status)
  DisableToolBarButton(#Main_Toolbar, #Main_Stop_Process, #False)
  StartThread(*ThreadData, @thWork())
EndProcedure
; IDE Options = PureBasic 5.71 LTS (MacOS X - x64)
; CursorPosition = 813
; FirstLine = 796
; Folding = -------
; EnableXP
; CompileSourceDirectory